
#ifndef INC_DISPLAYENGINE
#define INC_DISPLAYENGINE
#endif

#ifndef INC_GAMELOOPIFC
#include "GameLoopIfc.h"
#endif

struct BITMAP;

//----------------------------------------------------------------------------

struct ScreenRect
{
  ScreenRect() : x(0), y(0), w(0), h(0) {}
  ScreenRect( int ax, int ay, int aw, int ah ) : x(ax), y(ay), w(aw), h(ah) {}
  int x;
  int y;
  int w;
  int h;
  bool is_inside_rect( int bx, int by ) const
  {
    return x<bx && y<by && bx<x+w && by<y+h;
  }
};

//----------------------------------------------------------------------------

class DisplayEngine
{
public:
  enum UpdateMethod
  {
    UPDATE_NONE       = 0
  , UPDATE_SV_DOUBLE  = 1
  , UPDATE_VV_DOUBLE  = 2
  , UPDATE_SVV_TRIPLE = 3
  };

  DisplayEngine();
  ~DisplayEngine();
  void init( const char*ini_fn );
  BITMAP*const draw_buffer() const { return system_buffer ? system_buffer : active_buffer; }
  void prepare_draw_buffer();
  void display_draw_buffer();
  UpdateMethod getCurrentUpdateMethod() const { return update_method_; }
private:
  void allocate_and_clear_buffers( UpdateMethod );
  void destroy_all_buffers();
  void init_gfx();
  UpdateMethod update_method_;
  BITMAP* active_buffer;
  BITMAP* system_buffer;
  BITMAP* video_buffer1;
  BITMAP* video_buffer2;
};

//----------------------------------------------------------------------------

class ViewPortBase : public ViewPortIfc
{
public:
  ViewPortBase( const ScreenRect& );
  ~ViewPortBase();
  virtual void notify_chg_size();
  void chg_width_left( int w );
  void chg_width_right( int w );
  void chg_height_top( int h );
  void chg_height_bottom( int h );

protected:
  ScreenRect area;
};

//----------------------------------------------------------------------------

